@@ -1,4 +1,4 @@
-#    $Id: Nominet.pm,v 1.24 2019/07/13 15:25:03 pete Exp $
+#    $Id: Nominet.pm,v 1.25 2019/07/19 12:43:06 pete Exp $
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@ use constant EPP_XMLNS	=> 'urn:ietf:params:xml:ns:epp-1.0';
 use vars qw($Error $Code $Message);
 
 BEGIN {
-	our $VERSION = '0.03_04';
+	our $VERSION = '0.03_05';
 }
 
 # file-scoped lexicals
@@ -712,7 +712,10 @@ account like this to perform the registration.
 		'trad-name'   => 'Examples4u',
 		'type'        => 'LTD',
 		'co-no'	      => '12345678',
-		'opt-out'     => 'n',
+		'disclose'    => {
+			'org'  => 1,
+			'addr' => 0
+		},
 		'postalInfo'  => { loc => {
 			'name'  => 'Arnold N Other',
 			'org'   => 'Example Company',
@@ -757,7 +760,7 @@ sub create_contact {
 	# Extensions
 	my @spec = $self->spec ('contact-nom-ext');
 	my $obj  = $frame->addObject (@spec);
-	for my $field (qw/ trad-name type co-no opt-out /) {
+	for my $field (qw/ trad-name type co-no /) {
 		next unless ($contact->{$field});
 		my $name = $frame->createElement("contact-nom-ext:$field");
 		$name->appendText ($contact->{$field});
@@ -767,6 +770,19 @@ sub create_contact {
 	$extension->appendChild ($obj);
 	$frame->command->insertAfter ($extension, $frame->getCommandNode);
 
+	if (defined $contact->{disclose}) {
+		my $add = $frame->createElement ('contact:disclose');
+		$add->setAttribute('flag', '1');
+		for my $field (qw/org addr/) {
+			next unless $contact->{disclose}->{$field};
+			my $disc = $frame->createElement ("contact:$field");
+			$disc->setAttribute('type', 'loc');
+			$add->appendChild ($disc);
+		}
+		$frame->getNode('create')->getChildNodes->shift->appendChild ($add)
+			if $add->hasChildNodes;
+	}
+
 	my $response = $self->_send_frame ($frame);
 	return $Code == 1000 ? 1 : undef;
 }
@@ -961,7 +977,7 @@ sub modify_domain {
 =head2 Modify contacts
 
 To modify a contact, which includes aspects of the registrant such as
-the WHOIS opt-out etc., you will again need to create a hashref of the
+the disclose flags etc., you will again need to create a hashref of the
 changes like this:
 
 	my $changes = {
@@ -969,7 +985,6 @@ changes like this:
 		'type'        =>  'FCORP',
 		'trad-name'   =>  'American Industries',
 		'co-no'       =>  '99998888',
-		'opt-out'     =>  'N',
 		'postalInfo'  => {
 			'loc' => {
 				'name' => 'James Johnston',
@@ -983,7 +998,10 @@ changes like this:
 			}
 		},
 		'voice'	=>	'+1.77777776666',
-		'email'	=>	'jj@example.com'
+		'email'	=>	'jj@example.com',
+		'disclose' => {
+			'addr' => 1
+		}
 	};
 	my $res = $epp->update_contact ($changes) or die $epp->get_reason;
 
@@ -996,6 +1014,10 @@ There is also a convenience method C<modify_contact()> which takes the
 contact id as the first argument and the hashref of changes as the
 second argument.
 
+Note that due to an undocumented restriction in Nominet's EPP servers
+it is not possible to modify the disclose flags for both addr and org
+to different values in one request.
+
 =cut
 
 sub update_contact {
@@ -1073,6 +1095,16 @@ sub modify_contact {
 		$chg->appendChild ($elem);
 	}
 	if (defined $data->{disclose}) {
+		# Return an error if there's a mix of flags
+		my @flags = values (%{$data->{disclose}});
+		if ($#flags > 0 && $flags[0] != $flags[1]) {
+			$Error = "Nominet prohibits adding and removing disclosures " .
+				"in one action";
+			return;
+		}
+		# This doesn't need to be so complicated but it's staying this
+		# way in case Nominet decide to allow a mix of actions in the
+		# future.
 		my $add = $frame->createElement ('contact:disclose');
 		$add->setAttribute('flag', '1');
 		my $del = $frame->createElement ('contact:disclose');
@@ -1092,7 +1124,7 @@ sub modify_contact {
 	# Extensions
 	@spec = $self->spec ('contact-nom-ext');
 	$obj  = $frame->addObject (@spec);
-	for my $field (qw/ trad-name type co-no opt-out /) {
+	for my $field (qw/ trad-name type co-no /) {
 		next unless ($data->{$field});
 		my $name = $frame->createElement("contact-nom-ext:$field");
 		$name->appendText ($data->{$field});
@@ -1267,9 +1299,7 @@ sub _disclose_infData_to_hash {
 	my $flag = $disc->getAttribute('flag');
 	my $hash;
 	for my $child ($disc->getChildrenByTagName('*')) {
-		$hash->{$child->localname} = {
-			flag => $flag,
-		};
+		$hash->{$child->localname} = $flag;
 	}
 	return $hash;
 }
@@ -1296,7 +1326,7 @@ sub _merge_contact_infData {
 	my ($self, $old, $extra, $disclose) = @_;
 
 	my $extrahash = $self->_node_to_hash ($extra, ['type',
-	'co-no', 'opt-out', 'trad-name']);
+	'co-no', 'trad-name']);
 
 	for (keys %$extrahash) {
 		$old->{$_} = $extrahash->{$_};
